""" Code generation for explicitly specified tensors """

from dune.perftool.generation import (get_counted_variable,
                                      kernel_cached,
                                      instruction,
                                      temporary_variable,
                                      )

import pymbolic.primitives as prim
import numpy as np


def define_list_tensor(name, expr, visitor):
    for i, child in enumerate(expr.ufl_operands):
        instruction(assignee=prim.Subscript(prim.Variable(name), (i,)),
                    expression=visitor.call(child))


@kernel_cached
def pymbolic_list_tensor(expr, visitor):
    name = get_counted_variable("listtensor")
    temporary_variable(name,
                       shape=expr.ufl_shape,
                       dtype=np.float64,
                       )
    define_list_tensor(name, expr, visitor)
    return prim.Variable(name)
